#   -*- coding: utf-8 -*-
#   This file is part of Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import os.path
import sys

from fabric.api import local, lcd


def virtualenv_create():
    local("%s /usr/bin/virtualenv --no-site-packages virtualenv"
        % sys.executable, capture=False)


def install_requirements():
    local("virtualenv/bin/pip install -r test_requirements.txt", capture=False)
    local("virtualenv/bin/python setup.py develop", capture=False)
    symlink_python_apt()
    symlink_gtk2()


def symlink_python_apt():
    version_minor = sys.version_info[1]
    system_libdir = '/usr/lib/python2.%s/' % version_minor
    virtualenv_libdir = 'virtualenv/lib/python2.%s/' % version_minor
    _symlink(
        system_libdir + 'dist-packages/apt/',
        virtualenv_libdir + 'site-packages/apt')
    _symlink(
        system_libdir + 'dist-packages/apt_pkg.so',
        virtualenv_libdir + 'site-packages/apt_pkg.so')
    _symlink(
        system_libdir + 'dist-packages/apt_inst.so',
        virtualenv_libdir + 'site-packages/apt_inst.so')


def symlink_gtk2():
    minor = sys.version_info[1]

    def _symlink_libs(filenames):
        for filename in filenames:
            basename = os.path.basename(filename)
            _symlink(filename, 'virtualenv/lib/python2.%s/site-packages/%s' %
                (minor, basename))

    gtk_dir_preoneiric = '/usr/lib/pymodules/python2.%s/gtk-2.0' % minor
    gtk_dir_oneiric = '/usr/lib/python2.%s/dist-packages/gtk-2.0' % minor
    if os.path.exists(gtk_dir_preoneiric):
        # Ubuntu Lucid / Maverick / Natty setup
        _symlink_libs([
            gtk_dir_preoneiric,
            '/usr/lib/pymodules/python2.%s/cairo' % minor,
            ])
    else:
        # Ubuntu Oneiric setup
        _symlink_libs([
            gtk_dir_oneiric,
            '/usr/lib/python2.%s/dist-packages/cairo' % minor,
            ])

    pth_file_prenatty = '/usr/lib/pymodules/python2.%s/pygtk.pth' % minor
    pth_file_natty = '/usr/share/pyshared/pygtk.pth'
    if os.path.exists(pth_file_prenatty):
        # Ubuntu Lucid / Maveric setup
        _symlink_libs([
            pth_file_prenatty,
            '/usr/lib/pymodules/python2.%s/pygtk.py' % minor,
        ])
    elif os.path.exists(pth_file_natty):
        # Ubuntu Natty / Oneiric setup
        _symlink_libs([
            pth_file_natty,
            '/usr/lib/pymodules/python2.%s/cairo' % minor,
            '/usr/lib/python2.%s/dist-packages/glib' % minor,
            '/usr/lib/python2.%s/dist-packages/gobject' % minor,
            '/usr/lib/python2.%s/dist-packages/gtk-2.0' % minor,
            '/usr/lib/python2.%s/dist-packages/gtk-2.0-pysupport-compat.pth'
                % minor,
            '/usr/share/pyshared/pygtk.py',
            '/usr/lib/python2.%s/dist-packages/gtk-2.0/gio' % minor,
            '/usr/lib/python2.%s/dist-packages/gtk-2.0/LaunchpadIntegration'
                % minor,
        ])
    else:
        raise EnvironmentError("Unable to find system-wide gtk2 installation")


def _symlink(source, link_name):
    if not os.path.lexists(link_name):
        os.symlink(os.path.abspath(source), link_name)


def _get_or_pull_bzr_branch(repo, name, revision=None):
    if revision is None:
        revision = ""
    else:
        revision = "--revision={0}".format(revision)
    if not os.path.exists('branches'):
        os.mkdir('branches')
    branch_path = os.path.join("branches", name)
    if os.path.exists(branch_path):
        with lcd(branch_path):
            local("bzr pull {0} {1}".format(
                revision, repo), capture=False)
    else:
        local("bzr branch {0} {1} {2}".format(
            revision, repo, branch_path), capture=False)


def pull_required_branches():
    _get_or_pull_bzr_branch(
        "lp:~django-openid-auth/django-openid-auth/trunk",
        "django-openid-auth",
        revision=79)
    _symlink(
        "branches/django-openid-auth/django_openid_auth",
        "django_project/django_openid_auth")
    _get_or_pull_bzr_branch("lp:convoy", "convoy", revision=20)
    _symlink("branches/convoy/convoy", "django_project/convoy")
    _get_or_pull_bzr_branch(
        'lp:~canonical-ca-hackers/ubuntu-recommender/'
        'ubuntu-recommender-client', 'ubuntu-recommender-client')
    _symlink("branches/ubuntu-recommender-client/sreclient.py",
        "django_project/sreclient.py")

def bootstrap():
    virtualenv_create()
    install_requirements()
    pull_required_branches()


def clean():
    local("rm -rf branches/")
    local("rm -rf virtualenv/")
    local("rm -rf src/ubuntu_webcatalog.egg-info")
    local("rm -f webcatalog.db")
    links = ("django_openid_auth", )
    for link in links:
        local("rm -f django_project/{0}".format(link))
