#   -*- coding: utf-8 -*-
"""Fabric commands for deploying from the branches to testing server."""

from __future__ import absolute_import
from __future__ import with_statement

__metaclass__ = type
__all__ = [
    'deploy',
    ]

import os
import shutil
import tempfile

from fabric.api import env, run, local, put, cd, sudo
from fabric.contrib import files


def _format(s):
    """Format string using substitutions from fabric's env."""
    return s.format(**env)


def _run(cmd):
    run(_format(cmd))


def _local(cmd):
    local(_format(cmd))


def _sudo(cmd):
    sudo(_format(cmd))


def migrate_database(deploy_dir):
    path = os.path.join(
        deploy_dir, "ubuntu-webcatalog/sourcecode/framework")
    with cd(path):
        run("python manage.py syncdb --migrate --noinput")


def apache_graceful():
    sudo('service apache2 graceful', shell=False)


def deploy(config_branch, deploy_dir="/srv/uwc.staging.ubuntu.com/staging",
           production=False):
    env.cm = "/usr/lib/config-manager/cm.py"

    if not files.exists(env.cm):
        sudo("apt-get install --yes config-manager")

    env.config_branch = config_branch
    env.deploy_dir = deploy_dir
    env.build_dir = tempfile.mkdtemp()
    env.sourcedeps = _format("{build_dir}/cm/sourcedeps.conf")
    env.revno = local("bzr revno", capture=True).strip()

    _local("bzr branch {config_branch} {build_dir}/cm")

    _local("sed -i 's/bazaar.isd/bazaar.launchpad.net/g' {sourcedeps}")

    if not production:
        _local("sed -i '1 s/production/trunk/' {sourcedeps}")
        _local("sed -i '1 s/;revno=[0-9]\+$/;revno={revno}/' {sourcedeps}")
        _local("sed -i '2 s/;revno=[0-9]\+$//' {sourcedeps}")

    put(env.sourcedeps, deploy_dir)

    with cd(deploy_dir):
        _sudo("chown -R jenkins: ubuntu-webcatalog || true")

        if files.exists("ubuntu-webcatalog"):
            _run("{cm} update sourcedeps.conf")
        else:
            _run("{cm} build sourcedeps.conf")

        _run("rm sourcedeps.conf")

    shutil.rmtree(env.build_dir, ignore_errors=True)

    migrate_database(deploy_dir)

    _sudo("chown -R www-data: {deploy_dir}/ubuntu-webcatalog")

    apache_graceful()
