#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Admin classes for the Apps Directory."""

from __future__ import absolute_import

from django.contrib import admin
from webcatalog.models import (
    Application,
    ApplicationMedia,
    ApplicationWidget,
    Department,
    DistroSeries,
    Exhibit,
    Machine,
    ReviewStatsImport,
)

__metaclass__ = type
__all__ = [
    'ApplicationAdmin',
    'MachineAdmin',
]


class ApplicationWidgetInline(admin.TabularInline):
    model = ApplicationWidget.applications.through


class ApplicationAdmin(admin.ModelAdmin):
    list_display = ('package_name', 'name', 'comment', 'distroseries',
                    'wilson_score')
    search_fields = ('package_name', 'name', 'comment')
    list_filter = ('distroseries', 'imported_from_sca', 'is_latest',
                   'departments')
    inlines = [
        ApplicationWidgetInline,
    ]


class MachineAdmin(admin.ModelAdmin):
    search_fields = ('owner__username', 'hostname', 'uuid')
    list_display = ('hostname', 'uuid', 'owner')


class ExhibitAdmin(admin.ModelAdmin):
    list_display = ('package_names', 'published', 'display')


class DepartmentAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}


class ApplicationMediaAdmin(admin.ModelAdmin):
    search_fields = ('application__package_name',)
    list_filter = ('application__distroseries', 'media_type')
    list_display = ('__unicode__', 'distroseries', 'url')
    raw_id_fields = ('application',)

    def distroseries(self, obj):
        return obj.application.distroseries


class ApplicationWidgetAdmin(admin.ModelAdmin):
    list_display = ('name', )
    raw_id_fields = ('applications',)


class ReviewStatsImportAdmin(admin.ModelAdmin):
    list_display = ('distroseries', 'last_import')


admin.site.register(Application, ApplicationAdmin)
admin.site.register(ApplicationMedia, ApplicationMediaAdmin)
admin.site.register(ApplicationWidget, ApplicationWidgetAdmin)
admin.site.register(Department, DepartmentAdmin)
admin.site.register(DistroSeries)
admin.site.register(Exhibit, ExhibitAdmin)
admin.site.register(Machine, MachineAdmin)
admin.site.register(ReviewStatsImport, ReviewStatsImportAdmin)
