#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import url, patterns
from django.views.decorators.cache import never_cache
from django.views.decorators.gzip import gzip_page

from piston.resource import Resource
from webcatalog.api.handlers import (
    ListMachinesHandler,
    MachineHandler,
    MetadataHandler,
    PackagesHandler,
    ServerStatusHandler,
)
from webcatalog.auth import SSOOAuthAuthentication

auth = SSOOAuthAuthentication(realm="Ubuntu Software Center")


class CSRFExemptResource(Resource):
    """A Custom Resource that is csrf exempt"""
    def __init__(self, handler, authentication=None):
        super(CSRFExemptResource, self).__init__(handler, authentication)
        self.csrf_exempt = True

server_status_resource = never_cache(Resource(handler=ServerStatusHandler))
list_machines_resource = never_cache(Resource(handler=ListMachinesHandler,
                                              authentication=auth))
machine_resource = never_cache(CSRFExemptResource(handler=MachineHandler,
                                                  authentication=auth))
packages_resource = never_cache(CSRFExemptResource(handler=PackagesHandler,
                                                   authentication=auth))
metadata_resource = gzip_page(Resource(MetadataHandler))

urlpatterns = patterns(
    '',
    # get status of the service (usually just "ok", might be "read-only")
    # to ensure we inform the user when he wants to write a review or
    # send a moderation request
    #  GET /1.0/server-status/
    url(r'^1.0/server-status/$', server_status_resource,
        name='wb-server-status'),
    #  GET /1.0/list-machines/
    url(r'^1.0/list-machines/$', list_machines_resource,
        name='wb-list-machines'),
    url(r'^1.0/machine/(?P<uuid>[-\w]+)/$',
        machine_resource, name='wb-machine'),
    url(r'^1.0/packages/(?P<uuid>[-\w]+)/$',
        packages_resource, name='wb-packages'),
    url(r'^1.0/apps-meta/(?P<distroseries>\w+)/$', metadata_resource,
        name='wb-metadata-distro'),
    url(r'^1.0/apps-meta(/(?P<distroseries>\w+))?/$', metadata_resource,
        name='wb-metadata-all'),
)
