#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to import review statistics for all distroseries."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = []

from optparse import make_option

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import NoArgsCommand


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option(
            '--download-all',
            action='store_true',
            dest='download_all',
            default=False,
            help='Download all ratings stats instead of stats since last '
                 'import.'),
    )

    def handle_noargs(self, **options):
        self.verbosity = int(options['verbosity'])
        for distroseries in settings.UBUNTU_SERIES_FOR_VERSIONS.values():
            self.output("Importing ratings stats for {0}\n".format(
                distroseries), 1)
            call_command('import_ratings_stats', distroseries,
                         verbosity=self.verbosity,
                         download_all=options['download_all'])

    def output(self, message, level=None, flush=False):
        if hasattr(self, 'stdout'):
            if level is None or self.verbosity >= level:
                self.stdout.write(message)
            if flush:
                self.stdout.flush()
