#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Management command to import exhibits from Software Center Agent's API."""

from __future__ import (
    absolute_import,
)

import json
import urllib
from urlparse import urlparse

from django.conf import settings
from django.core.management.base import BaseCommand

from webcatalog.models import (
    DistroSeries,
    Exhibit,
)

__metaclass__ = type
__all__ = [
    'Command',
]


class Command(BaseCommand):
    help = "Import exhibits from Software Center Agent's API."
    verbosity = 0

    def handle(self, *args, **options):
        url = '%sexhibits/en/' % settings.SCA_API_URL
        # call api and get exhibits
        response = urllib.urlopen(url)
        if response.code != 200:
            raise Exception("Couldn't connect to server at %s" % url)
        exhibits = json.loads(response.read())

        for exhibit in exhibits:
            args = {'published': True}
            for field in ['package_names', 'html', 'banner_url', 'click_url']:
                args[field] = exhibit.get(field, '')

            args['weight'] = exhibit.get('weight', 0)

            # Munge banner_url into html
            banner_url = args['banner_url']
            if banner_url:
                parsed = urlparse(banner_url)
                args['html'] = args['html'].replace(parsed.path, banner_url)

            instance, created = Exhibit.objects.get_or_create(
                sca_id=exhibit['id'], defaults=args)

            if not created:
                Exhibit.objects.filter(pk=instance.pk).update(**args)

            distroseries = [DistroSeries.objects.get_or_create(**ds)[0]
                            for ds in exhibit.get('distroseries', [])]

            to_remove = instance.distroseries.exclude(
                pk__in=[x.id for x in distroseries])
            instance.distroseries.remove(*to_remove)
            instance.distroseries.add(*distroseries)

        # Unpublish other exhibits
        Exhibit.objects.filter(published=True).exclude(
            sca_id__in=[xibit['id'] for xibit in exhibits]).update(
                published=False)
