#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""configglue schema for the Apps Directory."""

from configglue import schema
from django_configglue.schema import schemas
from oops_dictconfig.configglue_options import OopsOption

DjangoSchema = schemas.get('1.5.1')


class WebCatalogSchema(DjangoSchema):
    """Config options specific to the web catalog."""
    # default section
    extra_pythonpath = schema.ListOption(item=schema.StringOption())
    pgconnect_timeout = schema.IntOption(default=10)
    should_serve_https = schema.BoolOption()

    class oops_wsgi(schema.Section):
        oopses = OopsOption(default={
            'publishers': [{
                'type': 'datedir',
                'error_dir': 'oopses',
                'inherit_id': True,
                'new_only': False,
            }],
        })

    class openid(schema.Section):
        openid_sso_server_url = schema.StringOption()
        openid_create_users = schema.BoolOption()
        openid_update_details_from_sreg = schema.BoolOption()
        openid_launchpad_teams_mapping = schema.DictOption()
        openid_sreg_extra_fields = schema.ListOption(
            item=schema.StringOption())
        openid_launchpad_staff_teams = schema.ListOption(
            item=schema.StringOption())

    class webcatalog(schema.Section):
        serve_site_media = schema.BoolOption(default=True)
        sca_api_url = schema.StringOption()
        disk_apt_cache_location = schema.StringOption()
        default_distro = schema.StringOption()
        page_batch_size = schema.IntOption(default=20)
        preload_api_service_roots = schema.BoolOption()
        oauth_data_store = schema.StringOption(
            default='webcatalog.models.oauthtoken.DataStore')
        convoy_root = schema.StringOption()
        featured_apps = schema.ListOption(
            item=schema.StringOption(), default=[])
        number_top_rated_apps = schema.IntOption(default=8)
        screenshots_base_url = schema.StringOption(
            default='http://screenshots.ubuntu.com/')
        ubuntu_series_for_versions = schema.DictOption(
            item=schema.StringOption())
        thumbnail_hosts = schema.ListOption(
            item=schema.StringOption(), default=[
                'sc.staging.ubuntu.com',
                'developer.staging.ubuntu.com',
                'software-center.ubuntu.com',
                'myapps.developer.ubuntu.com',
            ])

    class google(schema.Section):
        google_analytics_id = schema.StringOption()
        secondary_google_analytics_id = schema.StringOption()

    class logging(schema.Section):
        webapp_logging_config = schema.StringOption()

    class preflight(schema.Section):
        preflight_base_template = schema.StringOption(
            default="webcatalog/base.html")
        preflight_groups = schema.ListOption(item=schema.StringOption())

    class rnr(schema.Section):
        rnr_service_root = schema.StringOption(
            default="http://reviews.ubuntu.com/reviews/api/1.0")
        reviews_cache_timeout = schema.IntOption(default=60 * 15)

    class sso_api(schema.Section):
        sso_api_service_root = schema.StringOption()
        sso_api_auth_username = schema.StringOption()
        sso_api_auth_password = schema.StringOption()
        sso_api_identity_prefix = schema.StringOption()
        sso_auth_mode_no_ubuntu_sso_plaintext_only = schema.BoolOption()
        token_cache_expiry_hours = schema.IntOption(default=4)

    class email(schema.Section):
        noreply_from_address = schema.StringOption()

    class recommender(schema.Section):
        rec_service_root = schema.StringOption(
            default="http://rec.ubuntu.com/api/1.0")
        num_recommended_apps = schema.IntOption(default=4)

    class celery(schema.Section):
        broker_backend = schema.StringOption(default='memory')
        celery_result_backend = schema.StringOption(default='memory')
        celery_always_eager = schema.BoolOption()
        celery_eager_propagates_exceptions = schema.BoolOption()
        broker_url = schema.StringOption()
        celery_imports = schema.ListOption(item=schema.StringOption())
        celery_ignore_result = schema.BoolOption()
        celery_disable_rate_limits = schema.BoolOption(default=True)
        celerybeat_scheduler = schema.StringOption()
