from time import sleep

from celery.task import task
from django.core.management import call_command


@task(name="webcatalog.tasks.import_exhibits")
def import_exhibits():
    """Import all exhibits"""
    call_command('import_exhibits')


@task(name="webcatalog.tasks.import_sca")
def import_sca():
    """Import all data from the Software Center Agent (MyApps)"""
    call_command('import_sca_apps')


@task(name="webcatalog.tasks.import_app_install_data")
def import_app_install_data():
    """Import all data from app-install-data and apt-cache"""
    call_command('import_all_app_install_data', verbosity=0)


@task(name="webcatalog.tasks.import_ratings_stats")
def import_ratings_stats():
    """Import ratings data since the last import"""
    call_command('import_all_ratings_stats', verbosity=0)


@task(name="webcatalog.tasks.import_ratings_stats_all")
def import_ratings_stats_all():
    """Import all ratings data"""
    call_command('import_all_ratings_stats', verbosity=0, download_all=True)


@task(name="webcatalog.tasks.check_all_latest")
def check_all_latest():
    """Update the 'is_latest' bit on all Applications"""
    call_command('check_all_latest')


@task(name="webcatalog.tasks.cleanup_sessions")
def cleanup_sessions():
    """Remove stale sessions from the DB"""
    call_command('cleanup', 'django_session', verbosity=0)


@task(name="webcatalog.tasks.cleanup_nonces")
def cleanup_nonces():
    """Remove stale OAuth nonces from the DB"""
    call_command('cleanup', 'webcatalog_nonce', verbosity=0)


@task(name="webcatalog.tasks.fail")
def fail():
    """Sleep for 5 seconds, then raise an error"""
    sleep(5)
    raise ZeroDivisionError('Intentionally failed')
