#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for department filters."""

from __future__ import absolute_import

from webcatalog.tests.factory import TestCaseWithFactory
from webcatalog.department_filters import (
    category_filter,
    package_name_filter,
    section_filter
)

__metaclass__ = type
__all__ = [
    'DepartmentFilterTestCase',
]


class DepartmentFilterTestCase(TestCaseWithFactory):
    def test_package_name_filter(self):
        dept_filter = package_name_filter('a*$')
        app1 = self.factory.make_application(package_name='aaaaa')
        self.assertTrue(dept_filter(app1))

        app2 = self.factory.make_application(package_name='aaabaa')
        self.assertFalse(dept_filter(app2))

    def test_category_filter(self):
        dept_filter = category_filter(['foo', 'bar'])
        app = self.factory.make_application()
        self.assertFalse(dept_filter(app))
        app.categories = 'foo;bin'
        self.assertTrue(dept_filter(app))
        app.categories = 'foobin'
        self.assertFalse(dept_filter(app))
        app.categories = ''
        self.assertFalse(dept_filter(app))

    def test_section_filter(self):
        dept_filter = section_filter(['foo', 'bar'])
        app = self.factory.make_application()

        self.assertFalse(dept_filter(app))
        app.section = 'foo'
        self.assertTrue(dept_filter(app))
        app.section = 'foobin'
        self.assertFalse(dept_filter(app))
        app.section = ''
        self.assertFalse(dept_filter(app))
