#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test cases for oauth models."""

from webcatalog.models import (
    DataStore,
    Nonce,
)
from webcatalog.tests.factory import TestCaseWithFactory


class DataStoreTestCase(TestCaseWithFactory):

    def test_lookup_nonce_creates_nonce(self):
        token, consumer = self.factory.make_oauth_token_and_consumer()

        result = DataStore().lookup_nonce(
            consumer.oauth_consumer(), token.oauth_token(),
            'unique-nonce')

        self.assertFalse(result)
        self.assertEqual(1, Nonce.objects.filter(
            token=token, consumer=consumer,
            nonce='unique-nonce').count())

    def test_lookup_nonce_exists(self):
        nonce = self.factory.make_nonce(nonce_string='unique')
        token = nonce.token
        consumer = nonce.consumer

        result = DataStore().lookup_nonce(
            consumer.oauth_consumer(), token.oauth_token(),
            'unique')

        self.assertTrue(result)

    def test_lookup_nonce_uniquish(self):
        # If a nonce for the given consumer/token doesn't
        # exist, it's considered uniquish and valid.
        nonce = self.factory.make_nonce(nonce_string='uniquish-nonce')
        token, consumer = self.factory.make_oauth_token_and_consumer()

        result = DataStore().lookup_nonce(
            consumer.oauth_consumer(), token.oauth_token(),
            'uniquish-nonce')

        self.assertFalse(result)
        self.assertEqual(1, Nonce.objects.filter(
            token=token, consumer=consumer,
            nonce='uniquish-nonce').count())
