#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Apps Directory pep-8 compliance tests."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'WebCatalogPep8TestCase',
]

import os
import pep8
from collections import defaultdict
from unittest import TestCase

import webcatalog


class PackagePep8TestCase(TestCase):
    maxDiff = None
    packages = []
    exclude = ['migrations']

    def setUp(self):
        self.errors = {}
        self.pep8style = pep8.StyleGuide(
            counters=defaultdict(int),
            doctest='',
            exclude=self.exclude,
            filename=['*.py'],
            ignore=[],
            messages=self.errors,
            repeat=True,
            select=[],
            show_pep8=False,
            show_source=False,
            max_line_length=79,
            quiet=0,
            statistics=False,
            testsuite='',
            verbose=0,
        )

    def test_all_code(self):
        for package in self.packages:
            self.pep8style.input_dir(os.path.dirname(package.__file__))
        self.assertEqual(self.pep8style.options.report.total_errors, 0)


class WebCatalogPep8TestCase(PackagePep8TestCase):
    packages = [webcatalog]
