from django.conf import settings
from django.test import TestCase
from mock import patch

from webcatalog.models import Exhibit
from webcatalog.tasks import (
    fail,
    import_exhibits,
)


class ImportExhbitsTaskTestCase(TestCase):
    @patch('webcatalog.management.commands.import_exhibits.urllib.urlopen')
    def test_success(self, mock_urlopen):
        """Test that the ``import_exhibits`` task runs with no errors"""
        fd = mock_urlopen.return_value
        fd.code = 200
        fd.read.return_value = """[{
        "package_names": "foobar",
        "banner_url": "http://example.com/exhibits/foobar.png",
        "distroseries": [{"version": "12.04", "code_name": "precise"}],
        "html": "<div><div>Hello world!</div></div>",
        "published": true,
        "date_created": "2012-06-11 17:22:45",
        "id": 2
        }]"""
        expected_url = '%sexhibits/en/' % settings.SCA_API_URL

        result = import_exhibits.delay()

        self.assertIsNone(result.get())
        self.assertTrue(result.successful())
        mock_urlopen.assertCalledWith(expected_url)
        self.assertEqual(1, Exhibit.objects.all().count())


class FailTaskTestCase(TestCase):
    @patch('webcatalog.tasks.sleep')
    def test_delays_and_fails_as_expected(self, mock_sleep):
        """Test that the ``fail`` task raises an Exception as expected"""
        self.assertRaises(ZeroDivisionError, fail.delay)
        mock_sleep.assertCalledWith(5)
