#   -*- coding: utf-8 -*-
#   This file is part of the Apps Directory
#   Copyright (C) 2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Url configuration for the Apps Directory."""

from __future__ import absolute_import

from django.conf.urls.defaults import patterns, include, url
from django.views.generic import TemplateView

__metaclass__ = type
__all__ = [
    'urlpatterns',
]


urlpatterns = patterns(
    'webcatalog.views',
    url(r'^$', 'index', name='wc-index'),
    url(r'^department/(?P<distro>[-.+\w]+)/(?P<dept_slug_or_id>[\w\d-]+)/$',
        'department_overview', name='wc-department'),
    url(r'^department/(?P<dept_slug_or_id>[\w\d-]+)/$', 'department_overview',
        name='wc-department'),
    url(r'^applications/$', 'application_list', name='wc-package-list'),
    url(r'^applications/screenshots/(?P<package_name>[-.+:\w]+)/$',
        'application_screenshots', name="wc-package-screenshots"),
    url(r'^applications/(?P<distro>[-.+\w]+)/(?P<package_name>[-.+:\w]+)/$',
        'application_detail', name="wc-package-detail"),
    # The following URL redirects to the series-specific URL above.
    url(r'^applications/(?P<package_name>[-.+:\w]+)/$', 'application_detail',
        name="wc-package-detail"),
    url(r'^applications/(?P<distro>[-.+\w]+)/(?P<package_name>[-.+:\w]+)/'
        r'reviews/$', 'application_reviews', name="wc-package-reviews"),
    url(r'^applications/(?P<distro>[-.+\w]+)/(?P<package_name>[-.+:\w]+)/'
        r'reviews-ajax/$', 'application_reviews', kwargs={'ajax': True},
        name="wc-package-reviews-ajax"),
    url(r'^recommends/(?P<package_name>[-.+:\w]+)/$',
        'application_recommends', name="wc-package-recommends"),
    url(r'^search/$', 'search', name="wc-search"),
    url(r'^search/(?P<distro>[-.+\w]+)/$', 'search', name="wc-search"),
    url(r'^tos/$', name="wc-tos",
        view=TemplateView.as_view(template_name="webcatalog/tos.html")),
    url(r'^tos/plain/$', name="wc-tos-plain",
        view=TemplateView.as_view(template_name="webcatalog/tos_plain.html")),
    url(r'^combo/$', 'combo_view', name='wc-combo'),
    url(r'^forbidden/$', 'forbidden', name='wc-forbidden'),
    url(r'^tasks/$', 'tasks', name='wc-tasks'),
    url(r'^task_status/(?P<task_id>[-\da-f]+)/$', 'task_status',
        name='wc-task-status'),

    (r'^api/', include('webcatalog.api.urls')),
    url(r'^die/$', 'die', name='debug-die'),
)
